
Function LMNetStatisticsServerGet2% (Server$, Options&, VB_StatServer As stat_server_0)

' Wrapper:  LMNetStatisticsServerGet2
'    File:  NETSTATS.BAS
' Purpose:  Retrieves, and optionally clears, operating
'           statistics for a server.

'   Data Structure:  stat_server_0
'            Level:  0 (must be 0)
' Associated Files:  NETSTATS.TXT

' Parameters:  Server - the name of the server on which to execute the
'                       command. A NULL string specifies the local computer.

'              Options - specifies the options flags.  Currently the only
'                       option supported is the Clear Statistics option.
'                       The constant STATSOPT_CLR defines this option.

'                       The option to clear the statistics allows automatic
'                       get and clear operations.  This allows an application
'                       compiling cumulative numbers to ensure that no data
'                       is missed in the time between the get operation and
'                       the clear operation.

'              VB_StatServer - stat_server structure in which to store the
'                       returned data.


' Variables used in the NetStatisticsGet2 API call
    Dim Service As String               ' Constant - name of service "Server"
    Dim Level As Integer                ' information level
    Dim BufferPointer As Long           ' pointer to LM buffer
    Dim BufferSize As Integer           ' buffer size
    Dim TotalBytesAvail As Integer      ' total bytes available

' Other variables
    Dim result As Integer           ' return value for API call

    
    Service = "Server"      ' used in LM API call to indicate Server

    Level = 0   ' designates information level. Only level 0 is currently
                ' supported.

' Create LM buffer and get size in BufferSize
    BufferPointer = CreateLMBuffer(FMT_stat_server_0, 1, BufferSize)

    If BufferPointer = 0& Then      ' error, unable to allocate buffer
        LMNetStatisticsServerGet2 = -1        ' bugbug - what value?
        Exit Function
    End If

' Call LM API function NetStatisticsGet2 to get data
    result = NetStatisticsGet2(Server, Service, 0, Level, Options, BufferPointer, BufferSize, TotalBytesAvail)

' check for error return

    If result <> NERR_Success Then       ' error occurred
        LMNetStatisticsServerGet2 = result            ' set return for function
        result = FreeLMBuffer(BufferPointer)    ' free the buffer memory
        Exit Function
    End If

' Copy data from LM buffer to user_info structure
    result = BufferToVBType(VB_StatServer, Len(VB_StatServer), BufferPointer, BufferSize, FMT_stat_server_0)
    
' check if error
    If result = NERR_Success Then    ' return OK
        LMNetStatisticsServerGet2 = FreeLMBuffer(BufferPointer)  ' free memory for LM buffer
    Else                ' error occurred, set return value
        LMNetStatisticsServerGet2 = result
        result = FreeLMBuffer(BufferPointer)  ' free memory for LM buffer
    End If

End Function

Function LMNetStatisticsServerGet (Server$, VB_StatServer As statistics_info_0)

' Wrapper:  LMNetStatisticsServerGet
'    File:  NETSTATS.BAS
' Purpose:  Retrieves, and optionally clears, operating
'           statistics for a server.

'   Data Structure:  statistics_ino_0
'            Level:  0 (must be 0)
' >>>I put this in {mkj}

' Parameters:  Server - the name of the server on which to execute the
'                       command. A NULL string specifies the local computer.

'              VB_StatServer - stat_server structure in which to store the
'                       returned data.


' Variables used in the NetStatisticsGet2 API call
    Dim BufferPointer As Long           ' pointer to LM buffer
    Dim BufferSize As Integer           ' buffer size
    Dim TotalBytesAvail As Integer      ' total bytes available
    Dim BytesReturned As Integer

' Other variables
    Dim result As Integer           ' return value for API call

    

' Create LM buffer and get size in BufferSize
    BufferPointer = CreateLMBuffer(FMT_statistics_info_0, 1, BufferSize)

    If BufferPointer = 0& Then      ' error, unable to allocate buffer
        LMNetStatisticsServerGet = -1        ' bugbug - what value?
        Exit Function
    End If

' Call LM API function NetStatisticsGet to get data
    result = NetStatisticsGet(Server, BufferPointer, BufferSize, BytesReturned, TotalBytesAvail)

' check for error return

    If result <> NERR_Success Then       ' error occurred
        LMNetStatisticsServerGet = result            ' set return for function
        result = FreeLMBuffer(BufferPointer)    ' free the buffer memory
        Exit Function
    End If

' Copy data from LM buffer to user_info structure
    result = BufferToVBType(VB_StatServer, Len(VB_StatServer), BufferPointer, BufferSize, FMT_statistics_info_0)
    
' check if error
    If result = NERR_Success Then    ' return OK
        LMNetStatisticsServerGet = FreeLMBuffer(BufferPointer)  ' free memory for LM buffer
    Else                ' error occurred, set return value
        LMNetStatisticsServerGet = result
        result = FreeLMBuffer(BufferPointer)  ' free memory for LM buffer
    End If

End Function

