
Function LMNetServerEnum2_L1% (Server$, VB_ServerInfo() As server_info_1, ServerType&, Domain$)

' Wrapper:  LMNetServerEnum2_L1
'    File:  SERVER.BAS
' Purpose:  Lists all servers of the specified type(s)
'           that are visible in the specified domain(s).
'           For example, an application can call this
'           wrapper to list all domain controllers only
'           or all SQL servers only.

'   Data Structure:  server_info_1
'            Level:  1
' Associated Files:  SERVER.TXT

' Parameters:  Server - the name of the server on which to execute the
'                       command. A NULL string specifies the local computer.

'              VB_ServerInfo - server_info array in which to store the
'                       returned data.  The array will be ReDimmed to the
'                       required size and will be one-based.

'              ServerType - specifies the type(s) of servers to enumerate.
'                       The ServerType is tested against the sv1_type
'                       element of each entry.

'              Domain - contains the name of the domain in which to
'                       enumerate servers of the specified type(s).
'                       If Domain is a NULL string, servers are enumerated
'                       for the primary domain, logon domain, and other
'                       domains.

' Variables used in the NetServerEnum2 API call
    Dim Level As Integer                        ' information level
    Dim BufferPointer As Long                   ' pointer to LM buffer
    Dim BufferSize As Integer                   ' buffer size
    Dim Entries As Integer                      ' entries returned
    Dim TotalAvail As Integer                   ' total entries available

' Other variables
    Dim result As Integer          ' return value for function calls


    Level = 1   ' Designates information level, cannot just change this
                ' value to change info level - structure name and constant
                ' name must also be changed (server_info_1 and
                ' FMT_server_info_1).  The function name (LMNetServerEnum2_L1)
                ' should also be changed.

    ReDim VB_ServerInfo(0)  ' set to no entries present

    If ERROR_MORE_DATA = 0 Then
        MsgBox "The constant ERROR_MORE_DATA is not defined and is required.", 48, "LMNetServerEnum2_L1 Error"
        LMNetServerEnum2_L1 = -1       ' bugbug - what value?
    End If

' Call NetServerEnum2 with a 0 length to get the total number
' of entries available.  This will be used to allocate the buffer.
    result = NetServerEnum2(Server, Level, 0&, 0, Entries, TotalAvail, ServerType, Domain)
    
    If result <> ERROR_MORE_DATA Then      ' unexpected return code
        LMNetServerEnum2_L1 = result              ' set return code
        Exit Function
    End If

' Create LM buffer and get size in BufferSize
    BufferPointer = CreateLMBuffer(FMT_server_info_1, TotalAvail, BufferSize)

    If BufferPointer = 0& Then      ' error, unable to allocate buffer
        LMNetServerEnum2_L1 = -1              ' bugbug - what value?
        Exit Function
    End If

' Call LM API function NetServerEnum2 to get data
    result = NetServerEnum2(Server, Level, BufferPointer, BufferSize, Entries, TotalAvail, ServerType, Domain)
    
' check for error return
    If result <> NERR_Success Then     ' error occurred
        LMNetServerEnum2_L1 = result              ' set return for function
        result = FreeLMBuffer(BufferPointer)   ' free the buffer memory
        Exit Function
    End If

' Set # of entries for user_info array (one-based)
    ReDim VB_ServerInfo(1 To Entries)

' Copy data from LM buffer to user_info array
    result = EnumBufferToVBArray(VB_ServerInfo(1), Len(VB_ServerInfo(1)), BufferPointer, BufferSize, FMT_server_info_1, Entries)
    
' check if error
    If result = NERR_Success Then    ' return OK
        LMNetServerEnum2_L1 = FreeLMBuffer(BufferPointer)  ' free memory for LM buffer
    Else                ' error occurred, set return value
        LMNetServerEnum2_L1 = result
        result = FreeLMBuffer(BufferPointer)  ' free memory for LM buffer
    End If

End Function

Function LMNetServerGetInfo_L3% (Server$, VB_ServerInfo As server_info_3)

' Wrapper:  LMNetServerGetInfo_L3
'    File:  SERVER.BAS
' Purpose:  Retrieves information about the specified
'           server.

'   Data Structure:  server_info_3
'            Level:  3
' Associated Files:  SERVER.TXT

' Parameters:  Server - the name of the server on which to execute the
'                       command. A NULL string specifies the local computer.

'              VB_ServerInfo - server_info structure in which to store the
'                       returned data.


' Variables used in the NetServerGetInfo API call
    Dim Level As Integer                        ' information level
    Dim BufferPointer As Long                   ' pointer to LM buffer
    Dim BufferSize As Integer                   ' buffer size
    Dim TotalBytesAvail As Integer              ' total bytes available

' Other variables
    Dim result As Integer          ' return value for API call

                                                               
    Level = 3   ' designates information level, cannot just change this
                ' value to change info level - structure name and constant
                ' name must also be changed (server_info_3 and
                ' FMT_server_info_3).

' Create LM buffer and get size in BufferSize
    BufferPointer = CreateLMBuffer(FMT_server_info_3, 1, BufferSize)

    If BufferPointer = 0& Then      ' error, unable to allocate buffer
        LMNetServerGetInfo_L3 = -1        ' bugbug - what value?
        Exit Function
    End If

' Call LM API function NetServerGetInfo to get data
    result = NetServerGetInfo(Server, Level, BufferPointer, BufferSize, TotalBytesAvail)

' check for error return

    If result <> NERR_Success Then       ' error occurred
        LMNetServerGetInfo_L3 = result          ' set return for function
        result = FreeLMBuffer(BufferPointer)    ' free the buffer memory
        Exit Function
    End If

' Copy data from LM buffer to user_info structure
    result = BufferToVBType(VB_ServerInfo, Len(VB_ServerInfo), BufferPointer, BufferSize, FMT_server_info_3)
    
' check if error
    If result = NERR_Success Then    ' return OK
        LMNetServerGetInfo_L3 = FreeLMBuffer(BufferPointer)  ' free memory for LM buffer
    Else                ' error occurred, set return value
        LMNetServerGetInfo_L3 = result
        result = FreeLMBuffer(BufferPointer)  ' free memory for LM buffer
    End If

End Function

Function LMNetServerGetInfo_L1 (Server$, VB_ServerInfo As server_info_1) As Integer

' Wrapper:  LMNetServerGetInfo_L1
'    File:  SERVER.BAS
' Purpose:  Retrieves information about the specified
'           server.

'   Data Structure:  server_info_1
'            Level:  1
' >>>I added in {mkj} <<<

' Parameters:  Server - the name of the server on which to execute the
'                       command. A NULL string specifies the local computer.

'              VB_ServerInfo - server_info structure in which to store the
'                       returned data.


' Variables used in the NetServerGetInfo API call
    Dim Level As Integer                        ' information level
    Dim BufferPointer As Long                   ' pointer to LM buffer
    Dim BufferSize As Integer                   ' buffer size
    Dim TotalBytesAvail As Integer              ' total bytes available

' Other variables
    Dim result As Integer          ' return value for API call

                                                               
    Level = 1   ' designates information level, cannot just change this
                ' value to change info level - structure name and constant
                ' name must also be changed (server_info_3 and
                ' FMT_server_info_3).

' Create LM buffer and get size in BufferSize
    BufferPointer = CreateLMBuffer(FMT_server_info_1, 1, BufferSize)

    If BufferPointer = 0& Then      ' error, unable to allocate buffer
        LMNetServerGetInfo_L1 = -1        ' bugbug - what value?
        Exit Function
    End If

' Call LM API function NetServerGetInfo to get data
    result = NetServerGetInfo(Server, Level, BufferPointer, BufferSize, TotalBytesAvail)

' check for error return

    If result <> NERR_Success Then       ' error occurred
        LMNetServerGetInfo_L1 = result          ' set return for function
        result = FreeLMBuffer(BufferPointer)    ' free the buffer memory
        Exit Function
    End If

' Copy data from LM buffer to user_info structure
    result = BufferToVBType(VB_ServerInfo, Len(VB_ServerInfo), BufferPointer, BufferSize, FMT_server_info_1)
    
' check if error
    If result = NERR_Success Then    ' return OK
        LMNetServerGetInfo_L1 = FreeLMBuffer(BufferPointer)  ' free memory for LM buffer
    Else                ' error occurred, set return value
        LMNetServerGetInfo_L1 = result
        result = FreeLMBuffer(BufferPointer)  ' free memory for LM buffer
    End If


End Function

Function LMNetServerGetInfo_L2% (Server$, VB_ServerInfo As server_info_2)

End Function

